@@ -35,7 +35,7 @@ gnmTerms <- function(formula, eliminate = NULL, data = NULL)
                         attr(fullTerms, "term.labels"))
         instLabels <- as.list(attr(fullTerms, "variables"))[inst + 1]
         termLabels[termLabels %in% instLabels] <- 
-            vapply(instLabels, eval, character(1))
+            vapply(instLabels, eval, envir = env, character(1))
         variables <- as.character(attr(fullTerms, "variables"))[-1]
         offsetLabels <- variables[attr(fullTerms, "offset")]
         response <- variables[attr(fullTerms, "response")][1][[1]]
@@ -49,9 +49,7 @@ gnmTerms <- function(formula, eliminate = NULL, data = NULL)
                     attr(fullTerms, "term.labels"))
     variables <- predvars <- as.list(attr(fullTerms, "variables"))[-1]
 
-    specials <- which(vapply(variables, function(x) {
-        length(x) > 1 && inherits(match.fun(x[[1]]), "nonlin")
-    }, TRUE))
+    specials <- which(vapply(variables, isSpecial, envir = env, TRUE))
     if (!length(specials)) {
         n <- length(termLabels)
         attributes(fullTerms) <-
@@ -81,7 +79,7 @@ gnmTerms <- function(formula, eliminate = NULL, data = NULL)
     inst[patch] <- 1
 
     nonsense <- tapply(inst, term, FUN = function(x)
-                   {!is.na(x) && !identical(as.integer(x), seq(x))})
+                   {!all(is.na(x)) && !identical(as.integer(x), seq(x))})
     if (any(nonsense))
         stop("Specified instances of ",
              paste(names(nonsense)[nonsense], ")"),
@@ -106,10 +104,10 @@ gnmTerms <- function(formula, eliminate = NULL, data = NULL)
 
     for (j in which(nonlinear)) {
         nonlinCall <- parse(text = unitLabels[[j]])[[1]]
-        args <- eval(nonlinCall,
-                            as.data.frame(data), environment(formula))
+        args <- eval(nonlinCall, c(data, as.list(getNamespace("gnm"))), 
+                     environment(formula))
         args <- c(args, nonlin.function = deparse(nonlinCall[[1]]),
-                  list(data = data))
+                  list(data = data, envir = environment(formula)))
         tmp <- do.call("nonlinTerms", args)
         unitLabels[[j]] <- tmp$unitLabels
         if (!identical(tmp$prefix, "#")) {
@@ -184,3 +182,14 @@ gnmTerms <- function(formula, eliminate = NULL, data = NULL)
                class = c("gnmTerms", "terms", "formula")))
     fullTerms
 }
+
+isSpecial <- function(x, envir){
+    if (length(x) == 1) return(FALSE)
+    # look for function in gnm first, then in environment of formula
+    fn <- try(match.fun(eval(x[[1]])), silent = TRUE)
+    if (inherits(fn, "try-error")){
+        # will error here if can't find function
+        fn <- match.fun(eval(x[[1]], envir = envir))
+    }
+    inherits(fn, "nonlin")
+}
